#pragma once

/*
Copyright © Glensound Electronics Ltd and individual contributors. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met: Redistributions of source code must retain the
above copyright notice, this list of conditions and the following disclaimer. Redistributions
in binary form must reproduce the above copyright notice, this list of conditions and the
following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Glensound nor the names of its contributors may be used to endorse or
promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS” AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Content based on github.com/github/opensource.guide used under the CC-BY-4.0 license.
*/

#include "GsTypes.h"


// Types that may be shared among different devices

namespace Gs
{

	enum class AnalogueInputMode : uint8_t
	{
		Reserved,
		Line,		// line level source
		Mic,		// unpowered microphone
		Phantom		// Phantom-powered microphone
	};

	
	enum class SignalTypeChoice : uint8_t
	{
		Auto,
		Analogue,
		Digital
	};

	
	enum class SignalType : uint8_t
	{
		Analogue,
		Digital
	};

	
	enum class Pairing : uint8_t
	{
		Mono,
		Stereo
	};

	
	enum class Mute : uint8_t
	{
		Unmuted,
		Muted
	};
	

	struct alignas(uint32_t) GainIndex
	{
		uint8_t command;
		uint8_t index;				// channel index
		uint8_t gainIndex;			// index to a gain table
		uint8_t reserved;			// set to zero
	};
	static_assert(sizeof(GainIndex) == 4, "Size");

	
	struct alignas(uint32_t) InputMode
	{
		uint8_t command;
		uint8_t index;				// channel index
		AnalogueInputMode mode;
		uint8_t reserved;			// set to zero
	};
	static_assert(sizeof(InputMode) == 4, "Size");

	
	struct alignas(uint32_t) SignalTypeMsg
	{
		Command command;
		uint8_t index;				// channel index
		SignalTypeChoice choice;
		uint8_t reserved;			// set to zero
	};
	static_assert(sizeof(InputChoice) == 4, "Size");

	
	struct alignas(uint32_t) ChannelPairing
	{
		Command command;
		uint8_t index;				// channel pair index
		Pairing pairing;
		uint8_t reserved;			// set to zero
	};
	static_assert(sizeof(ChannelPairing) == 4, "Size");
	
}
