#pragma once

/*
Copyright © Glensound Electronics Ltd and individual contributors. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met: Redistributions of source code must retain the
above copyright notice, this list of conditions and the following disclaimer. Redistributions
in binary form must reproduce the above copyright notice, this list of conditions and the
following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Glensound nor the names of its contributors may be used to endorse or
promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS” AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Content based on github.com/github/opensource.guide used under the CC-BY-4.0 license.
*/

#include "GsTypes.h"


namespace Gs
{

	// Standard preset types that can be applied individually
	//
	enum class PresetType : uint8_t
	{
		Device,
		ChannelNames,
		_end
	};
	
	// Device-specific preset types start from this value
	//
	constexpr auto cNextPresetType = (uint8_t)PresetType::_end;
	
	
	// Standard settings types that can be individually restored to factory defaults
	//
	enum class SettingsType : uint8_t
	{
		Device,
		ChannelNames,
		_end	
	};
	
	// Device-specific settings types start from this value
	//
	constexpr auto cNextSettingsType = (uint8_t)SettingsType::_end;
	
	
	// These values can be ORed together for use with Command::RestoreDefaults
	// Device-specific bits can also be added.
	//
	enum class SettingsPart : uint32_t
	{
		Device = 1 << (int)SettingsType::Device,
		ChannelNames = 1 << (int)SettingsType::ChannelNames
	};
	
	
	// Standard device preset data (PresetType::Device).
	// NOTE: Different structure than DeviceReport.
	//
	struct alignas(uint32_t) DevicePreset
	{
		AdcFilter	adcFilter;
		DacFilter	dacFilter;
		Srate		sampleRateChoice;
		ClockSource	clockSourceChoice;
	};
	
	
	// Channel names preset data (PresetType::ChannelNames).
	// Uses the ChannelNamesReport structure.

}
