#pragma once

/*
Copyright © Glensound Electronics Ltd and individual contributors. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met: Redistributions of source code must retain the
above copyright notice, this list of conditions and the following disclaimer. Redistributions
in binary form must reproduce the above copyright notice, this list of conditions and the
following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Glensound nor the names of its contributors may be used to endorse or
promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS” AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Content based on github.com/github/opensource.guide used under the CC-BY-4.0 license.
*/

#include "GsTypes.h"


namespace Gs
{
		
	// Standard report types are supported or reserved by all devices
	//
	enum class ReportId : uint8_t
	{
		All,							// (in a Query message only) full report query
		Preset = All,					// preset query, preset report or preset generation index
		Device,							// standard Device report
		ChannelNames,					// (legacy devices only) names of the Dante input and output channels
		InChannelNames = ChannelNames,	// names of the Dante input channels
		OutChannelNames,				// names of the Dante output channels
		_end
	};
	
	// Device-specific report types start at this value
	//
	constexpr auto cNextReportId = (uint8_t)ReportId::_end;


	union DeviceFlags
	{
		uint32_t value;
		struct
		{
			uint32_t link1 : 1;			// network interface 1 state (0 = down, 1 = up)
			uint32_t link2 : 1;			// network interface 2 state (0 = down, 1 = up)
			uint32_t psuState1 : 1;		// 0 = PSU1 off, 1 = on
			uint32_t psuState2 : 1;		// 0 = PSU2 off, 1 = on
			uint32_t poeState1 : 1;		// 0 = PoE1 off, 1 = on
			uint32_t poeState2 : 1;		// 0 = PoE2 off, 1 = on
			uint32_t reserved3 : 26;	// may be defined by specific devices
		};
	};

	
	// Standard device report
	//
	struct DeviceReport
	{
		uint32_t	sampleRate;			// actual, in Hz
		Srate		srateChoice;		// user choice
		ClockSource	clockSource;		// actual
		ClockSource	clockSourceChoice;	// user choice
		AdcFilter	adc;
		DacFilter	dac;
		uint8_t		reserved1;
		uint16_t	reserved2;
		DeviceFlags	flags;
	};
	static_assert(sizeof(DeviceReport) == 16, "Size");
	
	
	// Channel names report
	//
	struct ChannelNamesReport
	{
		// Pseudocode:
		
		// PascalString names[numChannels];
		// char padding[];	// size must be a multiple of 4
	};

}
