#pragma once

/*
Copyright © Glensound Electronics Ltd and individual contributors. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met: Redistributions of source code must retain the
above copyright notice, this list of conditions and the following disclaimer. Redistributions
in binary form must reproduce the above copyright notice, this list of conditions and the
following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Glensound nor the names of its contributors may be used to endorse or
promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS” AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Content based on github.com/github/opensource.guide used under the CC-BY-4.0 license.
*/

#include "../Common/GsStatus.h"
#include "../Common/GsPresets.h"
#include "../Common/GsCommands.h"


namespace Dark8Mai
{
	
	constexpr size_t cNumInputs = 8;
	constexpr size_t cNumDanteOutChannels = cNumInputs;		// device inputs go to Dante outputs


	struct Status : Gs::Status
	{
		uint8_t		outputMeters[cNumDanteOutChannels];		// Dante output meters (in Dante format)
		uint8_t		gains[cNumInputs];						// analogue input gain indices
		uint16_t	inputModes;								// 2 bits per input: enum AnalogueInputMode
		uint16_t	reserved;
	};
	static_assert(sizeof(Status) == 28, "Size");


	enum class Command : uint8_t
	{
		// 4-byte commands
		//
		GainIndex = Gs::cNext4byteCommand,	// message is GainIndexMsg
		InputMode							// message is InputModeMsg
	};


	struct GainIndexMsg
	{
		Command		command;	// Command::GainIndex
		uint8_t		index;		// channel index (0 to 7)
		GainIndex	gainIndex;	// Line: 0-7, Mic: 0-17, Mic+Phantom: 0-17
		uint8_t		reserved;
	};
	static_assert(sizeof(GainIndexMsg) == 4);


	struct InputModeMsg
	{
		Command				command;	// Command::InputMode
		uint8_t				index;		// channel index (0 to 7)
		AnalogueInputMode	mode;		// 0 = reserved, 1 = line, 2 = mic, 3 = phantom
		uint8_t				reserved;
	};
	static_assert(sizeof(InputModeMsg) == 4);

	
	// Preset types that can be applied individually
	//
	enum class PresetType : uint8_t
	{
		Inputs = Gs::cNextPresetType
	};
	
	
	// Settings types that can be reset individually with the RestoreDefaults command
	//
	enum class SettingsType : uint8_t
	{
		Inputs = Gs::cNextSettingsType
	};

	
	// Flags to be ORed to define a selection of settings to be restored
	//
	enum class SettingsPart : uint32_t
	{
		Inputs = 1 << (int)SettingsType::Inputs
	};

}
