--package.prepend_path("Gs")
local helpers = require("Helpers")
local fields = require("Fields")
local header = require("GsHeader")
local info = require("Info")
local config = require("Config")
local status = require("Status")
local report = require("Report")
local command = require("Command")
local device = require("Device")
local divine = require("Divine")
local spark = require("Spark")
local dark1616m = require("Dark1616M")
local dark8mai = require("Dark8Mai")


gsProto = Proto("GS",  "GS Protocol")

helpers.merge_tables(fields.GetFields(), gsProto.fields)
helpers.merge_tables(header.GetFields(), gsProto.fields)
helpers.merge_tables(info.GetFields(), gsProto.fields)
helpers.merge_tables(config.GetFields(), gsProto.fields)
helpers.merge_tables(status.GetFields(), gsProto.fields)
helpers.merge_tables(report.GetFields(), gsProto.fields)
helpers.merge_tables(command.GetFields(), gsProto.fields)
helpers.merge_tables(device.GetFields(), gsProto.fields)
helpers.merge_tables(divine.GetFields(), gsProto.fields)
helpers.merge_tables(spark.GetFields(), gsProto.fields)
helpers.merge_tables(dark1616m.GetFields(), gsProto.fields)
helpers.merge_tables(dark8mai.GetFields(), gsProto.fields)

info.OnFieldsRegistered()
report.OnFieldsRegistered()
device.OnFieldsRegistered()
divine.OnFieldsRegistered()
spark.OnFieldsRegistered()
dark1616m.OnFieldsRegistered()
dark8mai.OnFieldsRegistered()

gsProto.experts = helpers.GetExperts()


function gsProto.dissector(buffer, pinfo, tree)
	length = buffer:len()
	if length == 0 then return end
	
	pinfo.cols.protocol = gsProto.name

	local subtree = tree:add(gsProto, buffer(), "GS Protocol")

	if buffer:reported_length_remaining() < 16 then
		helpers.ErrorSize(tree)
		return 0
	end
	
	local opcode, payloadSize = header.Parse(gsProto, buffer, subtree)
	local payload = buffer(16)
	
	if (payloadSize ~= payload:len()) then
		helpers.ErrorSize(tree)
		return 0
	end
	
	if opcode == 1 then
		status.Parse(gsProto, payload, pinfo.src, subtree)
	elseif opcode == 3 then
		command.Parse(gsProto, payload, pinfo.dst, subtree)
	elseif opcode == 4 then
		info.Parse(gsProto, payload, pinfo.src, subtree)
	elseif opcode == 6 then
		config.Parse(gsProto, payload, subtree)
	elseif opcode == 10 then
		report.ParseReport(gsProto, payload, pinfo.src, subtree)
	elseif opcode == 11 then
		report.ParseQuery(gsProto, payload, subtree)
	end
end

local udp_port = DissectorTable.get("udp.port")
udp_port:add(41161, gsProto)
