local helpers = require("Helpers")
local fields = require("Fields")

local m =
{
	-- Status
	ioRepGen = ProtoField.uint8("gs.dark1616m.ioRepGen", "IO board rep. gen.", base.DEC),
	micRepGen = ProtoField.uint8("gs.dark1616m.micRepGen", "Mic board rep. gen.", base.DEC),
	lockStates = ProtoField.uint8("gs.dark1616m.lockStates", "AES3 lock states", base.HEX),
	lock1 = ProtoField.bool("gs.dark1616m.lock1", "AES3 In 1-2", 8, nil, 0x01),
	lock2 = ProtoField.bool("gs.dark1616m.lock2", "AES3 In 3-4", 8, nil, 0x02),
	lock3 = ProtoField.bool("gs.dark1616m.lock3", "AES3 In 5-6", 8, nil, 0x04),
	lock4 = ProtoField.bool("gs.dark1616m.lock4", "AES3 In 7-8", 8, nil, 0x08),
	lock5 = ProtoField.bool("gs.dark1616m.lock5", "AES3 In 9-10", 8, nil, 0x10),
	lock6 = ProtoField.bool("gs.dark1616m.lock6", "AES3 In 11-12", 8, nil, 0x20),
	lock7 = ProtoField.bool("gs.dark1616m.lock7", "AES3 In 13-14", 8, nil, 0x40),
	lock8 = ProtoField.bool("gs.dark1616m.lock8", "AES3 In 15-16", 8, nil, 0x80),
	selInputs = ProtoField.uint8("gs.dark1616m.selInputs", "Selected inputs", base.HEX),
	sel1 = ProtoField.uint8("gs.dark1616m.sel1", "In 1-2", base.DEC, nil, 0x01),
	sel2 = ProtoField.uint8("gs.dark1616m.sel2", "In 3-4", base.DEC, nil, 0x02),
	sel3 = ProtoField.uint8("gs.dark1616m.sel3", "In 5-6", base.DEC, nil, 0x04),
	sel4 = ProtoField.uint8("gs.dark1616m.sel4", "In 7-8", base.DEC, nil, 0x08),
	sel5 = ProtoField.uint8("gs.dark1616m.sel5", "In 9-10", base.DEC, nil, 0x10),
	sel6 = ProtoField.uint8("gs.dark1616m.sel6", "In 11-12", base.DEC, nil, 0x20),
	sel7 = ProtoField.uint8("gs.dark1616m.sel7", "In 13-14", base.DEC, nil, 0x40),
	sel8 = ProtoField.uint8("gs.dark1616m.sel8", "In 15-16", base.DEC, nil, 0x80),
	
	-- Commands/Reports

	-- Reports
	inChoices = ProtoField.uint16("gs.dark1616m.inChoices", "Input choices", base.HEX),
	ich1 = ProtoField.uint16("gs.dark1616m.ich1", "In 1-2 choice", base.DEC, nil, 0x0003),
	ich2 = ProtoField.uint16("gs.dark1616m.ich2", "In 3-4 choice", base.DEC, nil, 0x000c),
	ich3 = ProtoField.uint16("gs.dark1616m.ich3", "In 5-6 choice", base.DEC, nil, 0x0030),
	ich4 = ProtoField.uint16("gs.dark1616m.ich4", "In 7-8 choice", base.DEC, nil, 0x00c0),
	ich5 = ProtoField.uint16("gs.dark1616m.ich5", "In 9-10 choice", base.DEC, nil, 0x0300),
	ich6 = ProtoField.uint16("gs.dark1616m.ich6", "In 11-12 choice", base.DEC, nil, 0x0c00),
	ich7 = ProtoField.uint16("gs.dark1616m.ich7", "In 13-14 choice", base.DEC, nil, 0x3000),
	ich8 = ProtoField.uint16("gs.dark1616m.ich8", "In 15-16 choice", base.DEC, nil, 0xc000),
	inModes = ProtoField.uint32("gs.dark1616m.inModes", "Input modes", base.HEX),
	im1 = ProtoField.uint32("gs.dark1616m.im1", "In 1 mode", base.DEC, nil, 0x00000003),
	im2 = ProtoField.uint32("gs.dark1616m.im2", "In 2 mode", base.DEC, nil, 0x0000000c),
	im3 = ProtoField.uint32("gs.dark1616m.im3", "In 3 mode", base.DEC, nil, 0x00000030),
	im4 = ProtoField.uint32("gs.dark1616m.im4", "In 4 mode", base.DEC, nil, 0x000000c0),
	im5 = ProtoField.uint32("gs.dark1616m.im5", "In 5 mode", base.DEC, nil, 0x00000300),
	im6 = ProtoField.uint32("gs.dark1616m.im6", "In 6 mode", base.DEC, nil, 0x00000c00),
	im7 = ProtoField.uint32("gs.dark1616m.im7", "In 7 mode", base.DEC, nil, 0x00003000),
	im8 = ProtoField.uint32("gs.dark1616m.im8", "In 8 mode", base.DEC, nil, 0x0000c000),
	im9 = ProtoField.uint32("gs.dark1616m.im9", "In 9 mode", base.DEC, nil, 0x00030000),
	im10 = ProtoField.uint32("gs.dark1616m.im10", "In 10 mode", base.DEC, nil, 0x000c0000),
	im11 = ProtoField.uint32("gs.dark1616m.im11", "In 11 mode", base.DEC, nil, 0x00300000),
	im12 = ProtoField.uint32("gs.dark1616m.im12", "In 12 mode", base.DEC, nil, 0x00c00000),
	im13 = ProtoField.uint32("gs.dark1616m.im13", "In 13 mode", base.DEC, nil, 0x03000000),
	im14 = ProtoField.uint32("gs.dark1616m.im14", "In 14 mode", base.DEC, nil, 0x0c000000),
	im15 = ProtoField.uint32("gs.dark1616m.im15", "In 15 mode", base.DEC, nil, 0x30000000),
	im16 = ProtoField.uint32("gs.dark1616m.im16", "In 16 mode", base.DEC, nil, 0xc0000000),
	inPairings = ProtoField.uint8("gs.dark1616m.inPairings", "Input pairings", base.HEX),
	ip1 = ProtoField.uint8("gs.dark1616m.ip1", "In 1-2 pairing", base.DEC, nil, 0x01),
	ip2 = ProtoField.uint8("gs.dark1616m.ip2", "In 3-4 pairing", base.DEC, nil, 0x02),
	ip3 = ProtoField.uint8("gs.dark1616m.ip3", "In 5-6 pairing", base.DEC, nil, 0x04),
	ip4 = ProtoField.uint8("gs.dark1616m.ip4", "In 7-8 pairing", base.DEC, nil, 0x08),
	ip5 = ProtoField.uint8("gs.dark1616m.ip5", "In 9-10 pairing", base.DEC, nil, 0x10),
	ip6 = ProtoField.uint8("gs.dark1616m.ip6", "In 11-12 pairing", base.DEC, nil, 0x20),
	ip7 = ProtoField.uint8("gs.dark1616m.ip7", "In 13-14 pairing", base.DEC, nil, 0x40),
	ip8 = ProtoField.uint8("gs.dark1616m.ip8", "In 15-16 pairing", base.DEC, nil, 0x80),

	-- Command
}


function m.GetFields()
	local fields =
	{
		-- Status
		dark1616m_ioRepGen = m.ioRepGen,
		dark1616m_micRepGen = m.micRepGen,
		dark1616m_lockStates = m.lockStates,
		dark1616m_lock1 = m.lock1,
		dark1616m_lock2 = m.lock2,
		dark1616m_lock3 = m.lock3,
		dark1616m_lock4 = m.lock4,
		dark1616m_lock5 = m.lock5,
		dark1616m_lock6 = m.lock6,
		dark1616m_lock7 = m.lock7,
		dark1616m_lock8 = m.lock8,
		dark1616m_selInputs = m.selInputs,
		dark1616m_sel1 = m.sel1,
		dark1616m_sel2 = m.sel2,
		dark1616m_sel3 = m.sel3,
		dark1616m_sel4 = m.sel4,
		dark1616m_sel5 = m.sel5,
		dark1616m_sel6 = m.sel6,
		dark1616m_sel7 = m.sel7,
		dark1616m_sel8 = m.sel8,

		-- Commands/Reports

		-- Reports
		dark1616m_inChoices = m.inChoices,
		dark1616m_ich1 = m.ich1,
		dark1616m_ich2 = m.ich2,
		dark1616m_ich3 = m.ich3,
		dark1616m_ich4 = m.ich4,
		dark1616m_ich5 = m.ich5,
		dark1616m_ich6 = m.ich6,
		dark1616m_ich7 = m.ich7,
		dark1616m_ich8 = m.ich8,
		dark1616m_inModes = m.inModes,
		dark1616m_im1 = m.im1,
		dark1616m_im2 = m.im2,
		dark1616m_im3 = m.im3,
		dark1616m_im4 = m.im4,
		dark1616m_im5 = m.im5,
		dark1616m_im6 = m.im6,
		dark1616m_im7 = m.im7,
		dark1616m_im8 = m.im8,
		dark1616m_im9 = m.im9,
		dark1616m_im10 = m.im10,
		dark1616m_im11 = m.im11,
		dark1616m_im12 = m.im12,
		dark1616m_im13 = m.im13,
		dark1616m_im14 = m.im14,
		dark1616m_im15 = m.im15,
		dark1616m_im16 = m.im16,
		dark1616m_inPairings = m.inPairings,
		dark1616m_ip1 = m.ip1,
		dark1616m_ip2 = m.ip2,
		dark1616m_ip3 = m.ip3,
		dark1616m_ip4 = m.ip4,
		dark1616m_ip5 = m.ip5,
		dark1616m_ip6 = m.ip6,
		dark1616m_ip7 = m.ip7,
		dark1616m_ip8 = m.ip8,

		-- Command
	}
	
	return fields
end


local fieldSel1
local fieldSel2
local fieldSel3
local fieldSel4
local fieldSel5
local fieldSel6
local fieldSel7
local fieldSel8
local fieldIch1
local fieldIch2
local fieldIch3
local fieldIch4
local fieldIch5
local fieldIch6
local fieldIch7
local fieldIch8
local fieldIm1
local fieldIm2
local fieldIm3
local fieldIm4
local fieldIm5
local fieldIm6
local fieldIm7
local fieldIm8
local fieldIm9
local fieldIm10
local fieldIm11
local fieldIm12
local fieldIm13
local fieldIm14
local fieldIm15
local fieldIm16
local fieldIp1
local fieldIp2
local fieldIp3
local fieldIp4
local fieldIp5
local fieldIp6
local fieldIp7
local fieldIp8

function m.OnFieldsRegistered()
	fieldSel1 = Field.new("gs.dark1616m.sel1")
	fieldSel2 = Field.new("gs.dark1616m.sel2")
	fieldSel3 = Field.new("gs.dark1616m.sel3")
	fieldSel4 = Field.new("gs.dark1616m.sel4")
	fieldSel5 = Field.new("gs.dark1616m.sel5")
	fieldSel6 = Field.new("gs.dark1616m.sel6")
	fieldSel7 = Field.new("gs.dark1616m.sel7")
	fieldSel8 = Field.new("gs.dark1616m.sel8")
	fieldIch1 = Field.new("gs.dark1616m.ich1")
	fieldIch2 = Field.new("gs.dark1616m.ich2")
	fieldIch3 = Field.new("gs.dark1616m.ich3")
	fieldIch4 = Field.new("gs.dark1616m.ich4")
	fieldIch5 = Field.new("gs.dark1616m.ich5")
	fieldIch6 = Field.new("gs.dark1616m.ich6")
	fieldIch7 = Field.new("gs.dark1616m.ich7")
	fieldIch8 = Field.new("gs.dark1616m.ich8")
	fieldIm1 = Field.new("gs.dark1616m.im1")
	fieldIm2 = Field.new("gs.dark1616m.im2")
	fieldIm3 = Field.new("gs.dark1616m.im3")
	fieldIm4 = Field.new("gs.dark1616m.im4")
	fieldIm5 = Field.new("gs.dark1616m.im5")
	fieldIm6 = Field.new("gs.dark1616m.im6")
	fieldIm7 = Field.new("gs.dark1616m.im7")
	fieldIm8 = Field.new("gs.dark1616m.im8")
	fieldIm9 = Field.new("gs.dark1616m.im9")
	fieldIm10 = Field.new("gs.dark1616m.im10")
	fieldIm11 = Field.new("gs.dark1616m.im11")
	fieldIm12 = Field.new("gs.dark1616m.im12")
	fieldIm13 = Field.new("gs.dark1616m.im13")
	fieldIm14 = Field.new("gs.dark1616m.im14")
	fieldIm15 = Field.new("gs.dark1616m.im15")
	fieldIm16 = Field.new("gs.dark1616m.im16")
	fieldIp1 = Field.new("gs.dark1616m.ip1")
	fieldIp2 = Field.new("gs.dark1616m.ip2")
	fieldIp3 = Field.new("gs.dark1616m.ip3")
	fieldIp4 = Field.new("gs.dark1616m.ip4")
	fieldIp5 = Field.new("gs.dark1616m.ip5")
	fieldIp6 = Field.new("gs.dark1616m.ip6")
	fieldIp7 = Field.new("gs.dark1616m.ip7")
	fieldIp8 = Field.new("gs.dark1616m.ip8")
end


local function CmdName(cmd)
	if cmd == 5 then return "Input gain"
	elseif cmd == 6 then return "Input mode"
	elseif cmd == 7 then return "Input choice"
	elseif cmd == 8 then return "Pairing"
	end

	return "Unknown"
end


local function IchName(val)
	if val == 0 then return "Auto"
	elseif val == 1 then return "Analogue"
	elseif val == 2 then return "Digital"
	end

	return "Unknown"
end


-- Returns true if the status may be valid
function m.MaybeValidStatus(buffer)
	return buffer:len() == 40
end


function m.ParseStatus(buffer, tree)
	if not m.MaybeValidStatus(buffer) then
		helpers.ErrorSize(tree)
		return
	end

	tree = tree:add(buffer(), "DARK1616M Status")
	tree:add(m.ioRepGen, buffer(0, 1))
	tree:add(m.micRepGen, buffer(1, 1))
	tree:add(g_f.meters, buffer(4, 32))
	
	local val = buffer(36, 1)
	local t = tree:add(m.lockStates, val)
	t:add(m.lock1, val)
	t:add(m.lock2, val)
	t:add(m.lock3, val)
	t:add(m.lock4, val)
	t:add(m.lock5, val)
	t:add(m.lock6, val)
	t:add(m.lock7, val)
	t:add(m.lock8, val)
	
	val = buffer(37, 1)
	t = tree:add(m.selInputs, val)
	t:add(m.sel1, val):append_text(" (".. g_f.ADName(fieldSel1()()) ..")")
	t:add(m.sel2, val):append_text(" (".. g_f.ADName(fieldSel2()()) ..")")
	t:add(m.sel3, val):append_text(" (".. g_f.ADName(fieldSel3()()) ..")")
	t:add(m.sel4, val):append_text(" (".. g_f.ADName(fieldSel4()()) ..")")
	t:add(m.sel5, val):append_text(" (".. g_f.ADName(fieldSel5()()) ..")")
	t:add(m.sel6, val):append_text(" (".. g_f.ADName(fieldSel6()()) ..")")
	t:add(m.sel7, val):append_text(" (".. g_f.ADName(fieldSel7()()) ..")")
	t:add(m.sel8, val):append_text(" (".. g_f.ADName(fieldSel8()()) ..")")
end


-- Returns true if the report may be valid
function m.MaybeValidReport(buffer, reportSize, reportType)
	return 	(reportType == 4 and reportSize == 4 and buffer:len() >= 4) or
			(reportType == 5 and reportSize == 72 and buffer:len() >= 72)
end


-- Returns the number of consumed bytes
function m.ParseReport(proto, buffer, reportSize, reportType, tree)
	local size = buffer:len()

	if not m.MaybeValidReport(buffer, reportSize, reportType) then
		helpers.ErrorSize(tree)
		return size
	end

	if reportType == 4 then
		tree = tree:add(proto, buffer(0, reportSize), "DARK1616M inputs report")
		local bytes = buffer(0, 2)
		local t = tree:add_le(m.inChoices, bytes)
		t:add_le(m.ich1, bytes):append_text(" (".. IchName(fieldIch1()()) ..")")
		t:add_le(m.ich2, bytes):append_text(" (".. IchName(fieldIch2()()) ..")")
		t:add_le(m.ich3, bytes):append_text(" (".. IchName(fieldIch3()()) ..")")
		t:add_le(m.ich4, bytes):append_text(" (".. IchName(fieldIch4()()) ..")")
		t:add_le(m.ich5, bytes):append_text(" (".. IchName(fieldIch5()()) ..")")
		t:add_le(m.ich6, bytes):append_text(" (".. IchName(fieldIch6()()) ..")")
		t:add_le(m.ich7, bytes):append_text(" (".. IchName(fieldIch7()()) ..")")
		t:add_le(m.ich8, bytes):append_text(" (".. IchName(fieldIch8()()) ..")")
	elseif reportType == 5 then
		tree = tree:add(proto, buffer(0, reportSize), "DARK1616M mic inputs report")
		local t = tree:add(g_f.gains, buffer(0, 64))
		t:add(g_f.gains, buffer(0, 4)):prepend_text("CH1 ")
		t:add(g_f.gains, buffer(4, 4)):prepend_text("CH2 ")
		t:add(g_f.gains, buffer(8, 4)):prepend_text("CH3 ")
		t:add(g_f.gains, buffer(12, 4)):prepend_text("CH4 ")
		t:add(g_f.gains, buffer(16, 4)):prepend_text("CH5 ")
		t:add(g_f.gains, buffer(20, 4)):prepend_text("CH6 ")
		t:add(g_f.gains, buffer(24, 4)):prepend_text("CH7 ")
		t:add(g_f.gains, buffer(28, 4)):prepend_text("CH8 ")
		t:add(g_f.gains, buffer(32, 4)):prepend_text("CH9 ")
		t:add(g_f.gains, buffer(36, 4)):prepend_text("CH10 ")
		t:add(g_f.gains, buffer(40, 4)):prepend_text("CH11 ")
		t:add(g_f.gains, buffer(44, 4)):prepend_text("CH12 ")
		t:add(g_f.gains, buffer(48, 4)):prepend_text("CH13 ")
		t:add(g_f.gains, buffer(52, 4)):prepend_text("CH14 ")
		t:add(g_f.gains, buffer(56, 4)):prepend_text("CH15 ")
		t:add(g_f.gains, buffer(60, 4)):prepend_text("CH16 ")
		local bytes = buffer(64, 4)
		t = tree:add_le(m.inModes, bytes)
		t:add_le(m.im1, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm1()()) ..")")
		t:add_le(m.im2, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm2()()) ..")")
		t:add_le(m.im3, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm3()()) ..")")
		t:add_le(m.im4, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm4()()) ..")")
		t:add_le(m.im5, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm5()()) ..")")
		t:add_le(m.im6, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm6()()) ..")")
		t:add_le(m.im7, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm7()()) ..")")
		t:add_le(m.im8, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm8()()) ..")")
		t:add_le(m.im9, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm9()()) ..")")
		t:add_le(m.im10, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm10()()) ..")")
		t:add_le(m.im11, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm11()()) ..")")
		t:add_le(m.im12, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm12()()) ..")")
		t:add_le(m.im13, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm13()()) ..")")
		t:add_le(m.im14, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm14()()) ..")")
		t:add_le(m.im15, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm15()()) ..")")
		t:add_le(m.im16, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm16()()) ..")")
		local val = buffer(68, 1)
		t = tree:add(m.inPairings, val)
		t:add(m.ip1, val):append_text(" (".. g_f.PairingName(fieldIp1()()) ..")")
		t:add(m.ip2, val):append_text(" (".. g_f.PairingName(fieldIp2()()) ..")")
		t:add(m.ip3, val):append_text(" (".. g_f.PairingName(fieldIp3()()) ..")")
		t:add(m.ip4, val):append_text(" (".. g_f.PairingName(fieldIp4()()) ..")")
		t:add(m.ip5, val):append_text(" (".. g_f.PairingName(fieldIp5()()) ..")")
		t:add(m.ip6, val):append_text(" (".. g_f.PairingName(fieldIp6()()) ..")")
		t:add(m.ip7, val):append_text(" (".. g_f.PairingName(fieldIp7()()) ..")")
		t:add(m.ip8, val):append_text(" (".. g_f.PairingName(fieldIp8()()) ..")")
	end
	
	return reportSize
end


function m.ParseCommand(buffer, tree)
	local cmd = buffer(0, 1):uint()
	tree:append_text(" (".. CmdName(cmd) ..")")
	
	local byte1 = buffer(1, 1)
	local val1 = byte1:uint()
	local byte2 = buffer(2, 1)
	local val2 = byte2:uint()
	
	if cmd == 5 then
		tree:add(g_f.channel, byte1)
		tree:add(g_f.gain, byte2)
	elseif cmd == 6 then
		tree:add(g_f.channel, byte1)
		tree:add(g_f.mode, byte2):append_text(" (".. g_f.AnalogueInputModeName(val2) ..")")
	elseif cmd == 7 then
		tree:add(g_f.chPair, byte1)
		tree:add(g_f.choice, byte2):append_text(" (".. IchName(val2) ..")")
	elseif cmd == 8 then
		tree:add(g_f.chPair, byte1)
		tree:add(g_f.pairing, byte2):append_text(" (".. g_f.PairingName(val2) ..")")
	end
end

return m
