local helpers = require("Helpers")
local fields = require("Fields")

local m =
{
	-- Status
	inModes = ProtoField.uint16("gs.dark8mai.inModes", "Input modes", base.HEX),
	im1 = ProtoField.uint16("gs.dark8mai.im1", "In 1 mode", base.DEC, nil, 0x0003),
	im2 = ProtoField.uint16("gs.dark8mai.im2", "In 2 mode", base.DEC, nil, 0x000c),
	im3 = ProtoField.uint16("gs.dark8mai.im3", "In 3 mode", base.DEC, nil, 0x0030),
	im4 = ProtoField.uint16("gs.dark8mai.im4", "In 4 mode", base.DEC, nil, 0x00c0),
	im5 = ProtoField.uint16("gs.dark8mai.im5", "In 5 mode", base.DEC, nil, 0x0300),
	im6 = ProtoField.uint16("gs.dark8mai.im6", "In 6 mode", base.DEC, nil, 0x0c00),
	im7 = ProtoField.uint16("gs.dark8mai.im7", "In 7 mode", base.DEC, nil, 0x3000),
	im8 = ProtoField.uint16("gs.dark8mai.im8", "In 8 mode", base.DEC, nil, 0xc000),
}


function m.GetFields()
	local fields =
	{
		-- Status
		dark8mai_inModes = m.inModes,
		dark8mai_im1 = m.im1,
		dark8mai_im2 = m.im2,
		dark8mai_im3 = m.im3,
		dark8mai_im4 = m.im4,
		dark8mai_im5 = m.im5,
		dark8mai_im6 = m.im6,
		dark8mai_im7 = m.im7,
		dark8mai_im8 = m.im8,
	}
	
	return fields
end


local fieldIm1
local fieldIm2
local fieldIm3
local fieldIm4
local fieldIm5
local fieldIm6
local fieldIm7
local fieldIm8

function m.OnFieldsRegistered()
	fieldIm1 = Field.new("gs.dark8mai.im1")
	fieldIm2 = Field.new("gs.dark8mai.im2")
	fieldIm3 = Field.new("gs.dark8mai.im3")
	fieldIm4 = Field.new("gs.dark8mai.im4")
	fieldIm5 = Field.new("gs.dark8mai.im5")
	fieldIm6 = Field.new("gs.dark8mai.im6")
	fieldIm7 = Field.new("gs.dark8mai.im7")
	fieldIm8 = Field.new("gs.dark8mai.im8")
end


local function CmdName(cmd)
	if cmd == 5 then return "Input gain"
	elseif cmd == 6 then return "Input mode"
	end

	return "Unknown"
end


-- Returns true if the status may be valid
function m.MaybeValidStatus(buffer)
	return buffer:len() == 20
end


function m.ParseStatus(buffer, tree)
	if not m.MaybeValidStatus(buffer) then
		helpers.ErrorSize(tree)
		return
	end
	
	tree = tree:add(buffer(), "DARK8MAI Status")
	tree:add(g_f.meters, buffer(0, 8))
	local t = tree:add(g_f.gains, buffer(8, 8))
	t:add(g_f.gain, buffer(8, 1)):prepend_text("CH1 ")
	t:add(g_f.gain, buffer(9, 1)):prepend_text("CH2 ")
	t:add(g_f.gain, buffer(10, 1)):prepend_text("CH3 ")
	t:add(g_f.gain, buffer(11, 1)):prepend_text("CH4 ")
	t:add(g_f.gain, buffer(12, 1)):prepend_text("CH5 ")
	t:add(g_f.gain, buffer(13, 1)):prepend_text("CH6 ")
	t:add(g_f.gain, buffer(14, 1)):prepend_text("CH7 ")
	t:add(g_f.gain, buffer(15, 1)):prepend_text("CH8 ")

	local bytes = buffer(16, 2)
	t = tree:add_le(m.inModes, bytes)
	t:add_le(m.im1, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm1()()) ..")")
	t:add_le(m.im2, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm2()()) ..")")
	t:add_le(m.im3, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm3()()) ..")")
	t:add_le(m.im4, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm4()()) ..")")
	t:add_le(m.im5, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm5()()) ..")")
	t:add_le(m.im6, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm6()()) ..")")
	t:add_le(m.im7, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm7()()) ..")")
	t:add_le(m.im8, bytes):append_text(" (".. g_f.AnalogueInputModeName(fieldIm8()()) ..")")
end


function m.ParseCommand(buffer, tree)
	local cmd = buffer(0, 1):uint()
	tree:append_text(" (".. CmdName(cmd) ..")")
	
	local byte1 = buffer(1, 1)
	local byte2 = buffer(2, 1)
	local val2 = byte2:uint()
	
	if cmd == 5 then
		tree:add(g_f.channel, byte1)
		tree:add(g_f.gain, byte2)
	elseif cmd == 6 then
		tree:add(g_f.channel, byte1)
		tree:add(g_f.mode, byte2):append_text(" (".. g_f.AnalogueInputModeName(val2) ..")")
	end
end

return m
