local helpers = require("Helpers")
local fields = require("Fields")

local m =
{
	-- Status
	potValue = ProtoField.uint8("gs.divine.potValue", "Pot value", base.DEC),

	-- Command/Report
	dim = ProtoField.uint8("gs.divine.dim", "Solo dim", base.DEC),
	minSolo = ProtoField.uint8("gs.divine.minSolo", "Min solo", base.DEC),
	mixSel = ProtoField.uint8("gs.divine.mixSel", "Mix select", base.DEC),
	eqType = ProtoField.uint32("gs.divine.eqType", "EQ type", base.DEC),
	password = ProtoField.uint8("gs.divine.password", "Password", base.HEX),
	meterMode = ProtoField.uint8("gs.divine.meterMode", "Meter mode", base.DEC),
	volumeBoost = ProtoField.uint8("gs.divine.volumeBoost", "Volume boost", base.DEC),
	volumeLimit = ProtoField.uint8("gs.divine.volumeLimit", "Volume limit", base.DEC),
	minVolume = ProtoField.uint8("gs.divine.minVolume", "Min volume", base.DEC),
	lineup = ProtoField.uint8("gs.divine.lineup", "Lineup", base.DEC),
	powerSave = ProtoField.uint8("gs.divine.powerSave", "Power save", base.DEC),
	displayBrightness = ProtoField.uint8("gs.divine.displayBrightness", "Display brightness", base.DEC),
	speakerGroup = ProtoField.uint8("gs.divine.speakerGroup", "Speaker group", base.DEC),

	-- Report
	r_lfCut = ProtoField.bool("gs.divine.r_lfCut", "LF cut", 32, nil, 0x00000001),
	r_potEnable = ProtoField.bool("gs.divine.r_potEnable", "Pot enable", 32, nil, 0x00000002),
	r_mixSelEnable = ProtoField.bool("gs.divine.r_mixSelEnable", "Mix sel enable", 32, nil, 0x00000004),
	r_darkMode = ProtoField.bool("gs.divine.r_darkMode", "Dark mode", 32, nil, 0x00000008),
	r_rearButtonsEnable = ProtoField.bool("gs.divine.r_rearButtonsEnable", "Rear buttons enable", 32, nil, 0x00000010),
	r_rearPasswordEnable = ProtoField.bool("gs.divine.r_rearPasswordEnable", "Rear passw. enable", 32, nil, 0x00000020),
	r_phaseInvert = ProtoField.bool("gs.divine.r_phaseInvert", "Phase invert", 32, nil, 0x00000040),
	priorities = ProtoField.uint16("gs.divine.priorities", "Priorities", base.HEX),
	priority1 = ProtoField.uint16("gs.divine.priority1", "Priority 1", base.DEC, nil, 0x0007),
	priority2 = ProtoField.uint16("gs.divine.priority2", "Priority 2", base.DEC, nil, 0x0038),
	priority3 = ProtoField.uint16("gs.divine.priority3", "Priority 3", base.DEC, nil, 0x01c0),
	trims = ProtoField.uint32("gs.divine.trims", "Trims", base.HEX),
	delays = ProtoField.uint32("gs.divine.delays", "Delays", base.HEX),
	mixEnables = ProtoField.uint8("gs.divine.mixEnables", "Mix enables", base.HEX),
	filterEnables = ProtoField.uint8("gs.divine.filterEnables", "Filter enables", base.HEX),

	-- Command
	mixType = ProtoField.uint8("gs.divine.mixType", "Mix type", base.DEC),
	trim = ProtoField.uint8("gs.divine.trim", "Trim", base.DEC),
	priority = ProtoField.uint8("gs.divine.priority", "Priority", base.DEC),
}


function m.GetFields()
	local fields =
	{
		-- Status
		divine_potValue = m.potValue,

		-- Command/Report
		divine_dim = m.dim,
		divine_minSolo = m.minSolo,
		divine_mixSel = m.mixSel,
		divine_eqType = m.eqType,
		divine_password = m.password,
		divine_meterMode = m.meterMode,
		divine_volumeBoost = m.volumeBoost,
		divine_volumeLimit = m.volumeLimit,
		divine_minVolume = m.minVolume,
		divine_lineup = m.lineup,
		divine_powerSave = m.powerSave,
		divine_displayBrightness = m.displayBrightness,
		divine_speakerGroup = m.speakerGroup,

		-- Report
		divine_r_lfCut = m.r_lfCut,
		divine_r_potEnable = m.r_potEnable,
		divine_r_mixSelEnable = m.r_mixSelEnable,
		divine_r_darkMode = m.r_darkMode,
		divine_r_rearButtonsEnable = m.r_rearButtonsEnable,
		divine_r_rearPasswordEnable = m.r_rearPasswordEnable,
		divine_r_phaseInvert = m.r_phaseInvert,
		divine_priorities = m.priorities,
		divine_priority1 = m.priority1,
		divine_priority2 = m.priority2,
		divine_priority3 = m.priority3,
		divine_trims = m.trims,
		divine_delays = m.delays,
		divine_mixEnables = m.mixEnables,
		divine_filterEnables = m.filterEnables,

		-- Command
		divine_mixType = m.mixType,
		divine_trim = m.trim,
		divine_priority = m.priority,
	}
	
	return fields
end

local fieldEqType


function m.OnFieldsRegistered()
	fieldEqType = Field.new("gs.divine.eqType")
end


local function EqTypeName(eqType)
	if eqType == 0 then return "Off"
	elseif eqType == 1 then return "Natural"
	elseif eqType == 2 then return "Normal"
	elseif eqType == 3 then return "Pop"
	end
	
	return "Unknown"
end


local function CmdName(cmd)
	if cmd == 5 then return "Mix selection"
	elseif cmd == 6 then return "Mix enable"
	elseif cmd == 7 then return "Delay"
	elseif cmd == 8 then return "Trim"
	elseif cmd == 9 then return "Filter"
	elseif cmd == 10 then return "Priority"
	elseif cmd == 11 then return "Solo dim"
	elseif cmd == 12 then return "Min solo"
	elseif cmd == 13 then return "Lineup"
	elseif cmd == 14 then return "Volume"
	elseif cmd == 15 then return "Min volume"
	elseif cmd == 16 then return "Volume limit"
	elseif cmd == 17 then return "Boost"
	elseif cmd == 18 then return "EQ type"
	elseif cmd == 19 then return "LF cut"
	elseif cmd == 20 then return "Pot enable"
	elseif cmd == 21 then return "Mix sel enable"
	elseif cmd == 22 then return "Rear buttons enable"
	elseif cmd == 23 then return "Password enable"
	elseif cmd == 24 then return "Set password"
	elseif cmd == 25 then return "Dark mode"
	elseif cmd == 26 then return "Power save"
	elseif cmd == 27 then return "Meter mode"
	elseif cmd == 28 then return "Rear brightness"
	elseif cmd == 29 then return "Speaker group"
	elseif cmd == 30 then return "Peer mix select"
	elseif cmd == 31 then return "Peer volume"
	elseif cmd == 32 then return "Phase invert"
	end

	return "Unknown"
end


-- Returns true if the status may be valid
function m.MaybeValidStatus(buffer)
	return buffer:len() == 16
end


function m.ParseStatus(buffer, tree)
	if not m.MaybeValidStatus(buffer) then
		helpers.ErrorSize(tree)
		return
	end

	tree = tree:add(buffer(), "Divine Status")
	tree:add(g_f.generation, buffer(0, 1))
	tree:add(g_f.meters, buffer(4, 8))
	tree:add(m.potValue, buffer(12, 1))
	tree:add(g_f.volume, buffer(13, 1))
	
	local val = buffer(14, 1)
	tree:add(g_f.temperature, val):append_text(" (".. g_f.ToCelsius(val:int()) ..")")
end


-- Returns true if the report may be valid
function m.MaybeValidReport(buffer, reportSize, reportType)
	return reportType == 4 and reportSize == 36 and buffer:len() >= 36
end


-- Returns the number of consumed bytes
function m.ParseReport(proto, buffer, reportSize, reportType, tree)
	local size = buffer:len()

	if not m.MaybeValidReport(buffer, reportSize, reportType) then
		helpers.ErrorSize(tree)
		return size
	end
	
	tree = tree:add(proto, buffer(0, reportSize), "Divine Report")
	
	local bytes = buffer(0, 4)
	local t = tree:add_le(g_f.flags32, bytes)
	t:add_le(m.r_lfCut, bytes)
	t:add_le(m.r_potEnable, bytes)
	t:add_le(m.r_mixSelEnable, bytes)
	t:add_le(m.r_darkMode, bytes)
	t:add_le(m.r_rearButtonsEnable, bytes)
	t:add_le(m.r_rearPasswordEnable, bytes)
	t:add_le(m.r_phaseInvert, bytes)
	
	t = tree:add_le(m.eqType, buffer(4, 4))
	t:append_text(" (".. EqTypeName(fieldEqType()()) ..")")
	
	bytes = buffer(8, 2)
	t = tree:add_le(m.priorities, bytes)
	t:add_le(m.priority1, bytes)
	t:add_le(m.priority2, bytes)
	t:add_le(m.priority3, bytes)
	
	tree:add(m.dim, buffer(10, 1))
	tree:add(m.minSolo, buffer(11, 1))
	tree:add(m.trims, buffer(12, 4))
	tree:add(m.delays, buffer(16, 4))
	tree:add(m.password, buffer(20, 1))
	tree:add(m.meterMode, buffer(21, 1))
	tree:add(m.volumeBoost, buffer(22, 1))
	tree:add(m.volumeLimit, buffer(23, 1))
	tree:add(m.minVolume, buffer(24, 1))
	tree:add(m.lineup, buffer(25, 1))
	tree:add(m.powerSave, buffer(26, 1))
	tree:add(m.mixSel, buffer(27, 1))
	tree:add(m.mixEnables, buffer(28, 1))
	tree:add(m.filterEnables, buffer(29, 1))
	tree:add(m.displayBrightness, buffer(30, 1))
	tree:add(m.speakerGroup, buffer(31, 1))
	
	return reportSize
end


function m.ParseCommand(buffer, tree)
	local cmd = buffer(0, 1):uint()
	local name = CmdName(cmd)
	tree:append_text(" ("..name..")")
	
	local byte1 = buffer(1, 1)
	local val1 = byte1:uint()
	local byte2 = buffer(2, 1)
	local val2 = byte2:uint()

	if cmd == 5 or cmd == 30 then
		tree:add(m.mixSel, byte1)
	elseif cmd == 6 then
		tree:add(m.mixType, byte1)
		tree:add(g_f.enable, byte2):append_text(" ("..g_f.EnableName(val2)..")")
	elseif cmd == 7 then
		tree:add(g_f.channel, byte1)
		tree:add(g_f.delay, byte2)
	elseif cmd == 8 then
		tree:add(g_f.channel, byte1)
		tree:add(m.trim, byte2)
	elseif cmd == 9 then
		tree:add(g_f.channel, byte1)
		tree:add(m.filter, byte2)
	elseif cmd == 10 then
		tree:add(g_f.channel, byte1)
		tree:add(g_f.priority, byte2)
	elseif cmd == 11 then
		tree:add(m.dim, byte1)
	elseif cmd == 12 then
		tree:add(m.minSolo, byte1)
	elseif cmd == 13 then
		tree:add(m.lineup, byte1)
	elseif cmd == 14 or cmd == 31 then
		tree:add(g_f.volume, byte1)
	elseif cmd == 15 then
		tree:add(m.minVolume, byte1)
	elseif cmd == 16 then
		tree:add(m.volumeLimit, byte1)
	elseif cmd == 17 then
		tree:add(m.volumeBoost, byte1)
	elseif cmd == 18 then
		tree:add(m.mixType, byte1)
		tree:add(m.eqType, byte2)
	elseif (cmd >= 19 and cmd <= 23) or cmd == 25 or cmd == 32 then
		tree:add(g_f.enable, byte1)
	elseif cmd == 24 then
		tree:add(m.password, byte1)
	elseif cmd == 26 then
		tree:add(m.powerSave, byte1)
	elseif cmd == 27 then
		tree:add(m.meterMode, byte1)
	elseif cmd == 28 then
		tree:add(m.displayBrightness, byte1)
	elseif cmd == 29 then
		tree:add(m.speakerGroup, byte1)
	end
end

return m
