local helpers = require("Helpers")
local fields = require("Fields")

local m =
{
	-- Status
	uiRepGen = ProtoField.uint8("gs.spark.uiRepGen", "UI rep. gen.", base.DEC),
	micRepGen = ProtoField.uint8("gs.spark.micRepGen", "Mic in rep. gen.", base.DEC),
	leds = ProtoField.uint8("gs.spark.leds", "Leds", base.HEX),
	ledPgm = ProtoField.bool("gs.spark.ledPgm", "PGM", 8, nil, 0x01),
	ledTb1 = ProtoField.bool("gs.spark.ledTb1", "TB1", 8, nil, 0x02),
	ledTb2 = ProtoField.bool("gs.spark.ledTb2", "TB2", 8, nil, 0x04),
	ledTb3 = ProtoField.bool("gs.spark.ledTb3", "TB3", 8, nil, 0x08),
	ledPwr = ProtoField.bool("gs.spark.ledPwr", "Power", 8, nil, 0x10),
	
	-- Command/Report

	-- Report
	buttonModes = ProtoField.uint16("gs.spark.buttonModes", "Button modes", base.HEX),
	pgmButtonMode = ProtoField.uint16("gs.spark.pgmButMode", "PGM buttton mode", base.HEX, nil, 0x000f),
	tb1ButtonMode = ProtoField.uint16("gs.spark.tb1ButMode", "TB1 buttton mode", base.HEX, nil, 0x00f0),
	tb2ButtonMode = ProtoField.uint16("gs.spark.tb2ButMode", "TB2 buttton mode", base.HEX, nil, 0x0f00),
	tb3ButtonMode = ProtoField.uint16("gs.spark.tb3ButMode", "TB3 buttton mode", base.HEX, nil, 0xf000),
	potModes = ProtoField.uint16("gs.spark.potModes", "Pot modes", base.HEX),
	cue1PotMode = ProtoField.uint16("gs.spark.cue1PotMode", "Cue1 pot mode", base.HEX, nil, 0x0003),
	cue2PotMode = ProtoField.uint16("gs.spark.cue2PotMode", "Cue2 pot mode", base.HEX, nil, 0x000c),
	cue3PotMode = ProtoField.uint16("gs.spark.cue3PotMode", "Cue3 pot mode", base.HEX, nil, 0x0030),
	cue4PotMode = ProtoField.uint16("gs.spark.cue4PotMode", "Cue4 pot mode", base.HEX, nil, 0x00c0),
	hpPotMode = ProtoField.uint16("gs.spark.hpPotMode", "HP pot mode", base.HEX, nil, 0x0300),
	stPotMode = ProtoField.uint16("gs.spark.stPotMode", "Sidetone pot mode", base.HEX, nil, 0x0c00),
	auxPotMode = ProtoField.uint16("gs.spark.auxPotMode", "Aux pot mode", base.HEX, nil, 0x3000),
	bkgPotMode = ProtoField.uint16("gs.spark.bkgPotMode", "Bkg pot mode", base.HEX, nil, 0xc000),
	volPans = ProtoField.bytes("gs.spark.volPans", "Volumes/Pans"),
	cue1VolPan = ProtoField.uint16("gs.spark.cue1VolPan", "Cue1 vol/pan", base.HEX),
	cue2VolPan = ProtoField.uint16("gs.spark.cue2VolPan", "Cue2 vol/pan", base.HEX),
	cue3VolPan = ProtoField.uint16("gs.spark.cue3VolPan", "Cue3 vol/pan", base.HEX),
	cue4VolPan = ProtoField.uint16("gs.spark.cue4VolPan", "Cue4 vol/pan", base.HEX),
	hpVolPan = ProtoField.uint16("gs.spark.hpVolPan", "HP vol/pan", base.HEX),
	stVolPan = ProtoField.uint16("gs.spark.stVolPan", "Sidetone vol/pan", base.HEX),
	auxVolPan = ProtoField.uint16("gs.spark.auxVolPan", "Aux vol/pan", base.HEX),
	bkgVolPan = ProtoField.uint16("gs.spark.bkgVolPan", "Bkg vol/pan", base.HEX),
	pgmMuteEnables = ProtoField.uint8("gs.spark.pgmMuteEnables", "PGM mute enables", base.HEX),
	tb1MutesPgm = ProtoField.bool("gs.spark.tb1MutesPgm", "TB1 mutes PGM", 8, nil, 0x02),
	tb2MutesPgm = ProtoField.bool("gs.spark.tb2MutesPgm", "TB2 mutes PGM", 8, nil, 0x04),
	tb3MutesPgm = ProtoField.bool("gs.spark.tb3MutesPgm", "TB3 mutes PGM", 8, nil, 0x08),
	compressor = ProtoField.bool("gs.spark.compressor", "Compressor", 8, nil, 0x01),
	phantom = ProtoField.bool("gs.spark.phantom", "Phantom", 8, nil, 0x02),
	gainLock = ProtoField.bool("gs.spark.gainLock", "Gain lock", 8, nil, 0x04),
	phantomLock = ProtoField.bool("gs.spark.phantomLock", "Phantom lock", 8, nil, 0x08),

	-- Command
}


function m.GetFields()
	local fields =
	{
		-- Status
		spark_uiRepGen = m.uiRepGen,
		spark_micRepGen = m.micRepGen,
		spark_leds = m.leds,
		spark_ledPgm = m.ledPgm,
		spark_ledTb1 = m.ledTb1,
		spark_ledTb2 = m.ledTb2,
		spark_ledTb3 = m.ledTb3,
		spark_ledPwr = m.ledPwr,

		-- Command/Report

		-- Report
		spark_buttonModes = m.buttonModes,
		spark_pgmButtonMode = m.pgmButtonMode,
		spark_tb1ButtonMode = m.tb1ButtonMode,
		spark_tb2ButtonMode = m.tb2ButtonMode,
		spark_tb3ButtonMode = m.tb3ButtonMode,
		spark_potModes = m.potModes,
		spark_cue1PotMode = m.cue1PotMode,
		spark_cue2PotMode = m.cue2PotMode,
		spark_cue3PotMode = m.cue3PotMode,
		spark_cue4PotMode = m.cue4PotMode,
		spark_hpPotMode = m.hpPotMode,
		spark_stPotMode = m.stPotMode,
		spark_auxPotMode = m.auxPotMode,
		spark_bkgPotMode = m.bkgPotMode,
		spark_volPans = m.volPans,
		spark_cue1VolPan = m.cue1VolPan,
		spark_cue2VolPan = m.cue2VolPan,
		spark_cue3VolPan = m.cue3VolPan,
		spark_cue4VolPan = m.cue4VolPan,
		spark_hpVolPan = m.hpVolPan,
		spark_stVolPan = m.stVolPan,
		spark_auxVolPan = m.auxVolPan,
		spark_bkgVolPan = m.bkgVolPan,
		spark_pgmMuteEnables = m.pgmMuteEnables,
		spark_tb1MutesPgm = m.tb1MutesPgm,
		spark_tb2MutesPgm = m.tb2MutesPgm,
		spark_tb3MutesPgm = m.tb3MutesPgm,
		spark_compressor = m.compressor,
		spark_phantom = m.phantom,
		spark_gainLock = m.gainLock,
		spark_phantomLock = m.phantomLock,

		-- Command
	}
	
	return fields
end


local fieldPgmMode
local fieldTb1Mode
local fieldTb2Mode
local fieldTb3Mode
local fieldCue1Mode
local fieldCue2Mode
local fieldCue3Mode
local fieldCue4Mode
local fieldHpMode
local fieldStMode
local fieldAuxMde
local fieldBkgMode

function m.OnFieldsRegistered()
	fieldPgmMode = Field.new("gs.spark.pgmButMode")
	fieldTb1Mode = Field.new("gs.spark.tb1ButMode")
	fieldTb2Mode = Field.new("gs.spark.tb2ButMode")
	fieldTb3Mode = Field.new("gs.spark.tb3ButMode")
	fieldCue1Mode = Field.new("gs.spark.cue1PotMode")
	fieldCue2Mode = Field.new("gs.spark.cue2PotMode")
	fieldCue3Mode = Field.new("gs.spark.cue3PotMode")
	fieldCue4Mode = Field.new("gs.spark.cue4PotMode")
	fieldHpMode = Field.new("gs.spark.hpPotMode")
	fieldStMode = Field.new("gs.spark.stPotMode")
	fieldAuxMode = Field.new("gs.spark.auxPotMode")
	fieldBkgMode = Field.new("gs.spark.bkgPotMode")
end


local function CmdName(cmd)
	if cmd == 5 then return "Mic gain"
	elseif cmd == 6 then return "Phantom"
	elseif cmd == 7 then return "Compressor"
	elseif cmd == 8 then return "Pot volume"
	elseif cmd == 9 then return "Pot pan"
	elseif cmd == 10 then return "Button"
	elseif cmd == 11 then return "Button mode"
	elseif cmd == 12 then return "PGM mute enable"
	elseif cmd == 13 then return "Pot mode"
	elseif cmd == 14 then return "Mic gain lock"
	elseif cmd == 15 then return "Phantom lock"
	end

	return "Unknown"
end


local function PotName(val)
	if val == 0 then return "Cue1"
	elseif val == 1 then return "Cue2"
	elseif val == 2 then return "Cue3"
	elseif val == 3 then return "Cue4"
	elseif val == 4 then return "HP"
	elseif val == 5 then return "Sidetone"
	elseif val == 6 then return "Aux"
	elseif val == 7 then return "Bkg"
	end

	return "Unknown"
end


local function ButName(val)
	if val == 0 then return "PGM"
	elseif val == 1 then return "TB1"
	elseif val == 2 then return "TB2"
	elseif val == 3 then return "TB3"
	end

	return "Unknown"
end


-- Returns true if the status may be valid
function m.MaybeValidStatus(buffer)
	return buffer:len() == 28
end


function m.ParseStatus(buffer, tree)
	if not m.MaybeValidStatus(buffer) then
		helpers.ErrorSize(tree)
		return
	end

	tree = tree:add(buffer(), "Spark Status")
	tree:add(m.uiRepGen, buffer(0, 1))
	tree:add(m.micRepGen, buffer(1, 1))
	tree:add(g_f.meters, buffer(4, 20))
	
	local val = buffer(24, 1)
	local t = tree:add(m.leds, val)
	t:add(m.ledPgm, val)
	t:add(m.ledTb1, val)
	t:add(m.ledTb2, val)
	t:add(m.ledTb3, val)
	t:add(m.ledPwr, val)
	
	val = buffer(27, 1)
	tree:add(g_f.temperature, val):append_text(" (".. g_f.ToCelsius(val:int()) ..")")
end


-- Returns true if the report may be valid
function m.MaybeValidReport(buffer, reportSize, reportType)
	return 	(reportType == 4 and reportSize == 24 and buffer:len() >= 24) or
			(reportType == 5 and reportSize == 4 and buffer:len() >= 4)
end


-- Returns the number of consumed bytes
function m.ParseReport(proto, buffer, reportSize, reportType, tree)
	local size = buffer:len()

	if not m.MaybeValidReport(buffer, reportSize, reportType) then
		helpers.ErrorSize(tree)
		return size
	end

	if reportType == 4 then
		tree = tree:add(proto, buffer(0, reportSize), "Spark UI report")
		local bytes = buffer(0, 2)
		local t = tree:add_le(m.buttonModes, bytes)
		t:add_le(m.pgmButtonMode, bytes):append_text(" (".. g_f.ButtonModeName(fieldPgmMode()()) ..")")
		t:add_le(m.tb1ButtonMode, bytes):append_text(" (".. g_f.ButtonModeName(fieldTb1Mode()()) ..")")
		t:add_le(m.tb2ButtonMode, bytes):append_text(" (".. g_f.ButtonModeName(fieldTb2Mode()()) ..")")
		t:add_le(m.tb3ButtonMode, bytes):append_text(" (".. g_f.ButtonModeName(fieldTb3Mode()()) ..")")
		
		bytes = buffer(2, 2)
		t = tree:add_le(m.potModes, bytes)
		t:add_le(m.cue1PotMode, bytes):append_text(" (".. g_f.PotModeName(fieldCue1Mode()()) ..")")
		t:add_le(m.cue2PotMode, bytes):append_text(" (".. g_f.PotModeName(fieldCue2Mode()()) ..")")
		t:add_le(m.cue3PotMode, bytes):append_text(" (".. g_f.PotModeName(fieldCue3Mode()()) ..")")
		t:add_le(m.cue4PotMode, bytes):append_text(" (".. g_f.PotModeName(fieldCue4Mode()()) ..")")
		t:add_le(m.hpPotMode, bytes):append_text(" (".. g_f.PotModeName(fieldHpMode()()) ..")")
		t:add_le(m.stPotMode, bytes):append_text(" (".. g_f.PotModeName(fieldStMode()()) ..")")
		t:add_le(m.auxPotMode, bytes):append_text(" (".. g_f.PotModeName(fieldAuxMode()()) ..")")
		t:add_le(m.bkgPotMode, bytes):append_text(" (".. g_f.PotModeName(fieldBkgMode()()) ..")")
		
		bytes = buffer(4, 16)
		t = tree:add(m.volPans, bytes)
		t:add_le(m.cue1VolPan, bytes(0, 2)):append_text(g_f.ToVolumePan(bytes(0, 2)))
		t:add_le(m.cue2VolPan, bytes(2, 2)):append_text(g_f.ToVolumePan(bytes(2, 2)))
		t:add_le(m.cue3VolPan, bytes(4, 2)):append_text(g_f.ToVolumePan(bytes(4, 2)))
		t:add_le(m.cue4VolPan, bytes(6, 2)):append_text(g_f.ToVolumePan(bytes(6, 2)))
		t:add_le(m.hpVolPan, bytes(8, 2)):append_text(g_f.ToVolumePan(bytes(8, 2)))
		t:add_le(m.stVolPan, bytes(10, 2)):append_text(g_f.ToVolumePan(bytes(10, 2)))
		t:add_le(m.auxVolPan, bytes(12, 2)):append_text(g_f.ToVolumePan(bytes(12, 2)))
		t:add_le(m.bkgVolPan, bytes(14, 2)):append_text(g_f.ToVolumePan(bytes(14, 2)))
		
		bytes = buffer(20, 1)
		t = tree:add(m.pgmMuteEnables, bytes)
		t:add(m.tb1MutesPgm, bytes)
		t:add(m.tb2MutesPgm, bytes)
		t:add(m.tb3MutesPgm, bytes)

	elseif reportType == 5 then
		tree = tree:add(proto, buffer(0, reportSize), "Spark mic input report")
		tree:add(g_f.gain, buffer(0, 1))
		local bytes = buffer(1, 1)
		local t = tree:add(g_f.flags8, bytes)
		t:add(m.compressor, bytes)
		t:add(m.phantom, bytes)
		t:add(m.gainLock, bytes)
		t:add(m.phantomLock, bytes)
	end
	
	return reportSize
end


function m.ParseCommand(buffer, tree)
	local cmd = buffer(0, 1):uint()
	local name = CmdName(cmd)
	tree:append_text(" ("..name..")")
	
	local byte1 = buffer(1, 1)
	local val1 = byte1:uint()
	local byte2 = buffer(2, 1)
	local val2 = byte2:uint()
	
	if cmd == 5 then
		tree:add(g_f.gain, byte1)
	elseif cmd == 6 or cmd == 7 or cmd == 14 or cmd == 15 then
		tree:add(g_f.enable, byte1)
	elseif cmd == 8 then
		tree:add(g_f.volume, byte1)
		tree:add(g_f.channel, byte2):append_text(" ("..PotName(val2)..")")
	elseif cmd == 9 then
		tree:add(g_f.pan, byte1)
		tree:add(g_f.channel, byte2):append_text(" ("..PotName(val2)..")")
	elseif cmd == 10 then
		tree:add(g_f.state, byte1):append_text(" ("..g_f.OffOnName(val1)..")")
		tree:add(g_f.button, byte2):append_text(" ("..ButName(val2)..")")
	elseif cmd == 11 then
		tree:add(g_f.button, byte1):append_text(" ("..ButName(val1)..")")
		tree:add(g_f.mode, byte2):append_text(" ("..g_f.ButtonModeName(val2)..")")
	elseif cmd == 12 then
		tree:add(g_f.button, byte1):append_text(" ("..ButName(val1)..")")
		tree:add(g_f.enable, byte2)
	elseif cmd == 13 then
		tree:add(g_f.channel, byte1):append_text(" ("..PotName(val1)..")")
		tree:add(g_f.mode, byte2):append_text(" ("..g_f.PotModeName(val2)..")")
	end
end

return m
